# -*- coding: utf-8 -*-
import hashlib
import shutil
from pathlib import Path

from .utils import convert_to_relative_paths


def test_convert_completezip(datadir, tmpdir):
    data_path = Path(str(tmpdir)) / 'col11405'
    shutil.copytree(str(datadir / 'completezip'), str(data_path))

    from litezip.completezip import convert_completezip
    data_struct = convert_completezip(data_path)

    def _keyed(s): return sorted({t[0]: t[1:] for t in s}.keys())

    from litezip.main import parse_litezip
    expected = parse_litezip(datadir / 'litezip')
    assert _keyed(data_struct) == _keyed(expected)

    relative_expected = convert_to_relative_paths(expected,
                                                  datadir / 'litezip')
    relative_data_struct = convert_to_relative_paths(data_struct, data_path)
    assert relative_data_struct == relative_expected

    def _hash_it(x):
        h = hashlib.sha1()
        h.update(x.open('rb').read())
        return h.hexdigest()
    hashes_expected = list([_hash_it(x) for _, x, __ in expected])
    hashes = list([_hash_it(x) for _, x, __ in data_struct])
    assert hashes == hashes_expected



# -*- coding: utf-8 -*-
import shutil
from pathlib import Path

import pytest

from ..utils import convert_to_relative_paths


@pytest.fixture
def data_path(datadir, tmpdir):
    """Copies the completezip to a place where we can write over it."""
    data_path = Path(str(tmpdir)) / 'col11405'
    shutil.copytree(str(datadir / 'completezip'), str(data_path))
    return data_path


def assert_equal(data_path, expected_path):
    """Asserts the data at `data_path` is equal to that at `expected_path`."""

    from litezip.main import parse_litezip
    data_struct = parse_litezip(data_path)
    expected = parse_litezip(expected_path)

    def _keyed(s): return sorted({t[0]: t[1:] for t in s}.keys())

    assert _keyed(data_struct) == _keyed(expected)

    relative_expected = convert_to_relative_paths(expected,
                                                  expected_path)
    relative_data_struct = convert_to_relative_paths(data_struct, data_path)
    assert relative_data_struct == relative_expected


def test_completezip2litezip(data_path, datadir):
    from litezip.cli.completezip2litezip import main
    retcode = main([str(data_path)])

    assert retcode == 0
    assert_equal(data_path, datadir / 'litezip')


def test_completezip2litezip_output(data_path, datadir, tmpdir):
    output_path = Path(str(tmpdir)) / 'col00000'

    from litezip.cli.completezip2litezip import main
    args = ['--output', str(output_path), str(data_path)]
    retcode = main(args)

    assert retcode == 0
    assert_equal(output_path, datadir / 'litezip')


def test_completezip2litezip_preexisting_output(data_path, datadir, tmpdir,
                                                capsys):
    output_path = Path(str(tmpdir.mkdir('col00000')))

    from litezip.cli.completezip2litezip import main
    args = ['--output', str(output_path), str(data_path)]
    retcode = main(args)

    assert retcode == 1

    out, err = capsys.readouterr()
    assert "ERROR: output-dir cannot exist prior to conversion" in out


def test_completezip2litezip_with_verbosity(data_path, datadir, capsys):
    from litezip.cli.completezip2litezip import main
    retcode = main(['-v', str(data_path)])

    assert retcode == 0

    out, err = capsys.readouterr()
    assert "DEBUG: removed " in out


def test_completezip2litezip_quiet_mode(data_path, datadir, capsys):
    from litezip.cli.completezip2litezip import main
    retcode = main(['-q', str(data_path)])

    assert retcode == 0

    out, err = capsys.readouterr()
    assert not out



# -*- coding: utf-8 -*-


def test_validate_litezip(datadir):
    args = [str(datadir / 'litezip')]

    from litezip.cli.validate import main
    retcode = main(args)

    assert retcode == 0


def test_validate_invalid_litezip(datadir, capsys):
    args = [str(datadir / 'invalid_litezip')]

    from litezip.cli.validate import main
    retcode = main(args)

    assert retcode == 1

    out, err = capsys.readouterr()
    assert not out


def test_validate_litezip_verbose_output(datadir, capsys):
    args = ['-v', str(datadir / 'invalid_litezip')]

    from litezip.cli.validate import main
    retcode = main(args)

    assert retcode == 1

    out, err = capsys.readouterr()
    assert 'collection.xml: 114:13 -- error: element' in out
    assert 'mux is not a valid identifier' in out
    assert 'mux/index.cnxml: 61:10 -- error: unknown element "foo"' in out


def test_validate_litezip_quiet_output(datadir, capsys):
    args = ['-q', str(datadir / 'invalid_litezip')]

    from litezip.cli.validate import main
    retcode = main(args)

    assert retcode == 1

    out, err = capsys.readouterr()
    assert not out



# -*- coding: utf-8 -*-
from pathlib import Path

import pytest


here = Path(__file__).parent
DATA_DIR = here / 'data'


@pytest.fixture
def datadir():
    """Returns the path to the data directory"""
    return DATA_DIR



# -*- coding: utf-8 -*-
import re

import cnxml

from .logger import logger


__all__ = (
    'is_valid_identifier',
    'validate_content',
    'validate_litezip',
)


VALID_ID_REGEX = re.compile("^(col\d{5,}\d*|m\d{4,})$")


def is_valid_identifier(id):
    """Validate that the given `id`."""
    return VALID_ID_REGEX.match(id) is not None


def validate_content(obj):
    """Runs the correct validator for the given `obj`ect."""
    from .main import Collection, Module
    validator = {
        Collection: cnxml.validate_collxml,
        Module: cnxml.validate_cnxml,
    }[type(obj)]
    return validator(obj.file)


def validate_litezip(struct):
    """Validate the given litezip as `struct`.
    Returns a list of validation messages.

    """
    msgs = []

    def _fmt_err(err): return "{}:{} -- {}: {}".format(*err)

    for obj in struct:
        if not is_valid_identifier(obj.id):
            msg = (obj.file.parent,
                   "{} is not a valid identifier".format(obj.id),)
            logger.info("{}: {}".format(*msg))
            msgs.append(msg)
        content_msgs = list([(obj.file, _fmt_err(err),)
                             for err in validate_content(obj)])
        for msg in content_msgs:
            logger.info("{}: {}".format(*msg))
        msgs.extend(content_msgs)
    return msgs



# -*- coding: utf-8 -*-
from litezip.logger import configure_logging


console_logging_config = {
    'version': 1,
    'formatters': {
        'cli': {
            'format': '%(levelname)-5.5s: %(message)s',
            },
    },
    'filters': {},
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'cli',
            'filters': [],
            'stream': 'ext://sys.stdout',
        },
    },
    'loggers': {
        'litezip': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': 0,
        },
    },
    'root': {
        'level': 'NOTSET',
        'handlers': [],
    },
}


def set_verbosity(verbose):
    config = console_logging_config.copy()
    if verbose is None:
        config['loggers']['litezip']['level'] = 'ERROR'
    elif verbose:
        config['loggers']['litezip']['level'] = 'DEBUG'
    else:  # quiet
        config['loggers']['litezip']['level'] = 100
    configure_logging(config)



# -*- coding: utf-8 -*-
import logging
from logging.config import dictConfig


__all__ = (
    'logger',
)

logger = logging.getLogger('litezip')


def configure_logging(config):
    """Configure logging given a dictified configuration."""
    dictConfig(config)



# -*- coding: utf-8 -*-
from pathlib import Path

from .logger import logger
from .main import parse_litezip


__all__ = (
    'convert_completezip',
)


def convert_completezip(path):
    """Converts a completezip file structure to a litezip file structure.
    Returns a litezip data structure.

    """
    for filepath in path.glob('**/index_auto_generated.cnxml'):
        filepath.rename(filepath.parent / 'index.cnxml')
        logger.debug('removed {}'.format(filepath))
    return parse_litezip(path)



# -*- coding: utf-8 -*-
from pathlib import Path

import pytest


def test_Module_struct(datadir):
    id = 'm40645'
    data_path = datadir / 'litezip' / id
    content = data_path / 'index.cnxml'
    resources = (data_path / 'Lab4 Fill Order.png',)

    from litezip.main import Module
    data_struct = Module(id, content, resources)

    assert len(data_struct) == 3
    assert data_struct.id == id
    assert data_struct.file == content
    assert data_struct.resources == (data_path / 'Lab4 Fill Order.png',)


def test_Collection_struct(datadir):
    id = 'col11405'
    data_path = datadir / 'litezip'
    content = data_path / 'collection.xml'
    resources = tuple()

    from litezip.main import Collection
    data_struct = Collection(id, content, resources)

    assert len(data_struct) == 3
    assert data_struct.id == id
    assert data_struct.file == content
    assert data_struct.resources == tuple()


def test_parse_module(datadir):
    module_id = 'm40645'
    data_path = datadir / 'litezip' / module_id

    from litezip.main import parse_module
    data_struct = parse_module(data_path)

    assert data_struct[0] == module_id
    assert data_struct[1] == data_path / 'index.cnxml'
    assert data_struct[2] == (data_path / 'Lab4 Fill Order.png',)


def test_parse_module_without_resources(datadir):
    module_id = 'm42304'
    data_path = datadir / 'litezip' / module_id

    from litezip.main import parse_module
    data_struct = parse_module(data_path)

    assert data_struct[0] == module_id
    assert data_struct[1] == data_path / 'index.cnxml'
    assert data_struct[2] == tuple()


def test_parse_module_raises_missing_file(tmpdir):
    module_id = 'm42000'
    data_path = Path(str(tmpdir.mkdir(module_id)))
    missing_file = data_path / 'index.cnxml'

    from litezip.main import parse_module
    from litezip.exceptions import MissingFile
    with pytest.raises(MissingFile) as exc_info:
        parse_module(data_path)

    assert missing_file == exc_info.value.args[0]


def test_parse_collection(datadir):
    col_id = 'col11405'
    data_path = datadir / 'litezip'

    from litezip.main import parse_collection
    data_struct = parse_collection(data_path)

    assert data_struct[0] == col_id
    assert data_struct[1] == data_path / 'collection.xml'
    assert data_struct[2] == tuple()


def test_parse_collection_raises_missing_file(tmpdir):
    col_id = 'col11405'
    data_path = Path(str(tmpdir.mkdir(col_id)))
    missing_file = data_path / 'collection.xml'

    from litezip.main import parse_collection
    from litezip.exceptions import MissingFile
    with pytest.raises(MissingFile) as exc_info:
        parse_collection(data_path)

    assert missing_file == exc_info.value.args[0]


def test_parse_litezip(datadir):
    data_path = datadir / 'litezip'

    from litezip.main import parse_litezip
    data_struct = parse_litezip(data_path)

    assert len(data_struct) == 8
    from litezip.main import Collection, Module
    col = Collection('col11405', data_path / 'collection.xml', tuple())
    assert data_struct[0] == col
    mods = [
        Module('m37154', data_path / 'm37154' / 'index.cnxml', tuple()),
        Module('m40646', data_path / 'm40646' / 'index.cnxml',
               tuple([data_path / 'm40646' / 'Photodiode.png'])),
    ]
    for mod in mods:
        assert mod in data_struct



# -*- coding: utf-8 -*-


def convert_to_relative_paths(struct, base):
    """Makes the given litezip `struct`'s `Path` objects relative to `base`.

    """
    def _rel(p): return p.relative_to(base)

    new_struct = []
    for obj in struct:
        new_obj = type(obj)(obj.id, _rel(obj.file),
                            tuple([_rel(y) for y in obj.resources]))
        new_struct.append(new_obj)
    return tuple(new_struct)



# -*- coding: utf-8 -*-
from __future__ import print_function
import argparse
import shutil
import sys
from pathlib import Path

from litezip import convert_completezip
from litezip.logger import logger
from .common import set_verbosity


def _arg_parser():
    """Factory for creating the argument parser"""
    description = "Converts a completezip to a litezip"
    parser = argparse.ArgumentParser(description=description)
    verbose_group = parser.add_mutually_exclusive_group()
    verbose_group.add_argument(
        '-v', '--verbose', action='store_true',
        dest='verbose', default=None,
        help="increase verbosity")
    verbose_group.add_argument(
        '-q', '--quiet', action='store_false',
        dest='verbose', default=None,
        help="print nothing to stdout or stderr")
    parser.add_argument(
        '-o', '--output-dir',
        help="Location to output, rather than converting inplace")
    parser.add_argument(
        'location',
        help="Location of the unpacked completezip")
    return parser


def main(argv=None):
    parser = _arg_parser()
    args = parser.parse_args(argv)

    set_verbosity(args.verbose)

    completezip_path = Path(args.location)
    output_dir = args.output_dir
    if output_dir:
        output_path = Path(output_dir)
        if output_path.exists():
            logger.error("output-dir cannot exist prior to conversion")
            return 1
        shutil.copytree(str(completezip_path), str(output_path))
        completezip_path = output_path

    struct = convert_completezip(completezip_path)

    return 0



# -*- coding: utf-8 -*-
from pathlib import Path

import pytest


def test_is_valid_identifier():
    from litezip.validate import is_valid_identifier as target
    assert target('m40646')
    assert target('col11405')
    assert not target('mi5')


def test_validate_collection(datadir):
    from litezip.main import parse_collection
    data_struct = parse_collection(datadir / 'litezip')

    from litezip.validate import validate_content
    errors = validate_content(data_struct)

    assert not errors


def test_validate_module(datadir):
    from litezip.main import parse_module
    data_struct = parse_module(datadir / 'litezip' / 'm40646')

    from litezip.validate import validate_content
    errors = validate_content(data_struct)

    assert not errors


def test_validate_litezip(datadir):
    from litezip.main import parse_litezip
    data_path = datadir / 'invalid_litezip'
    data_struct = parse_litezip(data_path)

    from litezip.validate import validate_litezip
    validation_msgs = validate_litezip(data_struct)

    expected = [
        (Path(data_path / 'collection.xml'),
         '114:13 -- error: element "para" from namespace '
         '"http://cnx.rice.edu/cnxml" not allowed in this context'),
        (Path(data_path / 'mux'), 'mux is not a valid identifier'),
        (Path(data_path / 'mux/index.cnxml'),
         '61:10 -- error: unknown element "foo" from namespace '
         '"http://cnx.rice.edu/cnxml"'),
    ]
    assert validation_msgs == expected



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages


setup_requires = (
    'pytest-runner',
    )
install_requires = (
    'cnxml',
    'lxml',
    'pathlib;python_version<="2.7"',
    )
tests_require = [
    'pytest',
    ]
extras_require = {
    'test': tests_require,
    }
description = "Connexions LiteZip Library"
with open('README.rst', 'r') as readme:
    long_description = readme.read()

setup(
    name='cnx-litezip',
    version='1.0.0',
    author='Connexions team',
    author_email='info@cnx.org',
    url="https://github.com/connexions/cnx-litezip",
    license='LGPL, See also LICENSE.txt',
    description=description,
    long_description=long_description,
    setup_requires=setup_requires,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=extras_require,
    test_suite='litezip.tests',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'litezip.tests': ['data/**/*.*'],
        },
    entry_points="""\
    [console_scripts]
    completezip2litezip = litezip.cli.completezip2litezip:main
    validate-litezip = litezip.cli.validate:main
    """,
    )



# -*- coding: utf-8 -*-
from collections import namedtuple

from lxml import etree

from .exceptions import MissingFile


__all__ = (
    'parse_collection',
    'parse_litezip',
    'parse_module',
    'Collection',
    'Module',
)


MODULE_FILENAME = 'index.cnxml'
COLLECTION_FILENAME = 'collection.xml'

Collection = namedtuple('Collection', 'id, file, resources')
Module = namedtuple('Module', 'id, file, resources')


COLLECTION_NSMAP = {
    "bib": "http://bibtexml.sf.net/",
    "c": "http://cnx.rice.edu/cnxml",
    "col": "http://cnx.rice.edu/collxml",
    "data": "http://www.w3.org/TR/html5/dom.html#custom-data-attribute",
    "datadev": "http://dev.w3.org/html5/spec/#custom",
    "dc": "http://purl.org/dc/elements/1.1/",
    "epub": "http://www.idpf.org/2007/ops",
    "lrmi": "http://lrmi.net/the-specification",
    "m": "http://www.w3.org/1998/Math/MathML",
    "md": "http://cnx.rice.edu/mdml",
    "mod": "http://cnx.rice.edu/#moduleIds",
    "qml": "http://cnx.rice.edu/qml/1.0",
    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
}


def _parse_collection_id(elm_tree):
    """Given the parsed xml to an `ElementTree`,
    parse the id from the content.

    """
    xpath = '//md:content-id/text()'
    nsmap = []
    return [x for x in elm_tree.xpath(xpath, namespaces=COLLECTION_NSMAP)][0]


def _find_resources(directory, excludes=[]):
    """Return a list of resource paths from the directory.
    Ignore records via the list of `excludes`,
    which are callables that take a file parameter (as a `Path` instance).

    """
    return [r for r in directory.glob('*')
            if True not in [e(r) for e in excludes]]


def parse_module(path):
    """Parse the file structure to a data structure given the path to
    a module directory.

    """
    id = path.name
    file = path / MODULE_FILENAME

    if not file.exists():
        raise MissingFile(file)

    excludes = [
        lambda filepath: filepath.name == MODULE_FILENAME,
    ]
    resources = tuple(_find_resources(path, excludes=excludes))

    return Module(id, file, resources)


def parse_collection(path):
    """Parse a file structure to a data structure given the path to
    a collection directory.

    """
    file = path / COLLECTION_FILENAME
    if not file.exists():
        raise MissingFile(file)
    id = _parse_collection_id(etree.parse(file.open()))

    excludes = [
        lambda filepath: filepath.name == COLLECTION_FILENAME,
        lambda filepath: filepath.is_dir(),
    ]
    resources = tuple(_find_resources(path, excludes=excludes))

    return Collection(id, file, resources)


def parse_litezip(path):
    """Parse a litezip file structure to a data structure given the path
    to the litezip directory.

    """
    struct = [parse_collection(path)]
    struct.extend([parse_module(x) for x in path.iterdir()
                   if x.is_dir() and x.name.startswith('m')])
    return tuple(struct)



# -*- coding: utf-8 -*-
import argparse
from pathlib import Path

from litezip import parse_litezip, validate_litezip
from .common import set_verbosity


def _arg_parser():
    """Factory for creating the argument parser"""
    description = "Converts a completezip to a litezip"
    parser = argparse.ArgumentParser(description=description)
    verbose_group = parser.add_mutually_exclusive_group()
    verbose_group.add_argument(
        '-v', '--verbose', action='store_true',
        dest='verbose', default=None,
        help="increase verbosity")
    verbose_group.add_argument(
        '-q', '--quiet', action='store_false',
        dest='verbose', default=None,
        help="print nothing to stdout or stderr")
    parser.add_argument(
        'location',
        help="Location of the unpacked completezip")
    return parser


def main(argv=None):
    parser = _arg_parser()
    args = parser.parse_args(argv)

    set_verbosity(args.verbose)

    struct = parse_litezip(Path(args.location))
    # Logging will provide output based on verbosity.
    errors = validate_litezip(struct)

    return errors and 1 or 0
